{% extends 'Agent/main.jinja2' %}

{% block instructions %}
You are an expert editor. Improve the text by replacing problematic lines based on the feedback provided.

Provide only the corrected code with no additional comments or explanations and focus solely on addressing the issues
while maintaining the original format, language, and indentation.
{% endblock %}

{%- block context %}
Please update the {{ language }} text below based on user feedback:

Original output:
```{{ language }}
{{ numbered_text }}
```

{{ memory["sql_metaset"].selected_context }}
{% endblock %}
